library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;
use ieee.std_logic_arith.all;

entity cq_max10_nco_sample is
    port
    (
        CLK_50MHZ_I : in std_logic; -- 50MHz Input
        DAC_CE_NO : out std_logic;
        DAC_D_O : out std_logic_vector(7 downto 0);
        SW_NI : in std_logic;
        LED_NO : out std_logic_vector(1 downto 0)
    );
end entity;

architecture rtl of cq_max10_nco_sample is
    signal r_reset_n : std_logic := '0';
    signal r_reset_delay : std_logic_vector(3 downto 0) := (others => '0');

    signal r_clock_counter : std_logic_vector(16 downto 0);
    signal r_clock_762hz : std_logic;

    signal r_sw_n_delay : std_logic_vector(1 downto 0);
    signal r_sw_n_or : std_logic;
    signal r_sw_n_or_delay : std_logic;
    signal r_sw_n_falling_edge : std_logic;
    signal r_sw_push_cnt : std_logic_vector(1 downto 0);

    component nco0 is
        port (
            clk       : in  std_logic                     := 'X';             -- clk
            clken     : in  std_logic                     := 'X';             -- clken
            phi_inc_i : in  std_logic_vector(31 downto 0) := (others => 'X'); -- phi_inc_i
            fsin_o    : out std_logic_vector(17 downto 0);                    -- fsin_o
            out_valid : out std_logic;                                        -- out_valid
            reset_n   : in  std_logic                     := 'X'              -- reset_n
        );
    end component nco0;
    signal r_nco0_phi_inc : std_logic_vector(31 downto 0);
    signal r_nco0_fsin_o : std_logic_vector(17 downto 0);
    signal r_nco0_out_valid : std_logic;
    signal r_nco0_fsin : std_logic_vector(7 downto 0);

    signal r_dac_clk_cnt : integer range 0 to 75;
    signal r_dac_cs : std_logic;
    signal r_dac_d_en : std_logic;
    signal r_dac_d : std_logic_vector(7 downto 0);

begin
    process(r_reset_n, CLK_50MHZ_I)
    begin
        if rising_edge(CLK_50MHZ_I) then
            r_reset_delay <= r_reset_delay + 1;
            r_reset_n <= r_reset_n or r_reset_delay(r_reset_delay'high);
        end if;
    end process;

    process(r_reset_n, CLK_50MHZ_I)
    begin
        if r_reset_n = '0' then
            r_clock_counter <= (others => '0');

        elsif rising_edge(CLK_50MHZ_I) then
            r_clock_counter <= r_clock_counter + 1;
        end if;
    end process;
    r_clock_762hz <= r_clock_counter(r_clock_counter'high);

    process(r_reset_n, r_clock_762hz)
    begin
        if r_reset_n = '0' then
            r_sw_n_delay <= (others => '0');
            r_sw_n_or_delay <= '0';

        elsif rising_edge(r_clock_762hz) then
            r_sw_n_delay <= r_sw_n_delay(r_sw_n_delay'high - 1 downto 0) & SW_NI;
            r_sw_n_or_delay <= r_sw_n_or;
        end if;
    end process;
    r_sw_n_or <= or_reduce(r_sw_n_delay);
    r_sw_n_falling_edge <= '1' when (r_sw_n_or_delay = '1') and (r_sw_n_or = '0') else '0';

    process(r_reset_n, r_clock_762hz)
    begin
        if r_reset_n = '0' then
            r_sw_push_cnt <= (others => '0');

        elsif rising_edge(r_clock_762hz) then
            if r_sw_n_falling_edge = '1' then
                r_sw_push_cnt <= r_sw_push_cnt + 1;
            end if;
        end if;
    end process;

    with (r_sw_push_cnt) select
        r_nco0_phi_inc <=
            X"00014F8B" when "00", -- 1kHz
            X"00029F16" when "01", -- 2kHz
            X"0003EEA2" when "10", -- 3kHz
            X"00053E2D" when others; -- 4kHz

    nco0_inst : nco0
    port map
    (
        clk       => CLK_50MHZ_I,
        clken     => '1',
        phi_inc_i => r_nco0_phi_inc,
        fsin_o    => r_nco0_fsin_o,
        out_valid => r_nco0_out_valid,
        reset_n   => r_reset_n
    );

    process(r_reset_n, CLK_50MHZ_I)
    begin
        if r_reset_n = '0' then
            r_nco0_fsin <= (others => '0');

        elsif rising_edge(CLK_50MHZ_I) then
            if r_nco0_out_valid = '1' then
                r_nco0_fsin <= r_nco0_fsin_o(17 downto 10);
            end if;
        end if;
    end process;

    process(r_reset_n, CLK_50MHZ_I)
    begin
        if r_reset_n = '0' then
            r_dac_clk_cnt <= 0;

        elsif rising_edge(CLK_50MHZ_I) then
            if r_dac_clk_cnt = 75 then
                r_dac_clk_cnt <= 0;
            else
                r_dac_clk_cnt <= r_dac_clk_cnt + 1;
            end if;
        end if;
    end process;

    r_dac_cs <= '0' when (r_dac_clk_cnt = 0) or (r_dac_clk_cnt = 1) else '1';
    r_dac_d_en <= '1' when (r_dac_clk_cnt = 2) else '0';

    process(r_reset_n, CLK_50MHZ_I)
    begin
        if r_reset_n = '0' then
            r_dac_d <= (others => '0');

        elsif rising_edge(CLK_50MHZ_I) then
            if r_dac_d_en = '1' then
                r_dac_d <= r_nco0_fsin + X"80";
            end if;
        end if;
    end process;

    DAC_CE_NO <= not r_dac_cs;
    DAC_D_O <= r_dac_d;

    LED_NO <= not r_sw_push_cnt;

end;
